/*
 * Copyright 2009 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.lisp;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.morilib.lisp.sos.LispType;

public class LispVector extends Datum {
	
	//
	private List<Datum> vec;
	
	
	private static final List<Datum> _EMPTY = Collections.emptyList();
	public  static final LispVector  EMPTY  = new LispVector(_EMPTY);
	
	
	public LispVector(List<Datum> vec) {
		if(vec == null) {
			throw new NullPointerException();
		}
		this.vec = vec;
	}
	
	
	public LispVector() {
		this(new ArrayList<Datum>());
	}
	
	
	public Datum get(int index) {
		return vec.get(index);
	}
	
	
	/*package*/ void set(int index, Datum d) {
		vec.set(index, d);
	}
	
	
	public int size() {
		return vec.size();
	}
	
	
	public Datum toConsList() {
		return LispUtils.listToCons(vec);
	}
	
	
	public String toString() {
		return LispUtils.getResult(this);
	}
	
	
	public boolean isTypeVector() {
		return true;
	}
	
	
	public List<Datum> getList() {
		return Collections.unmodifiableList(vec);
	}
	
	/* (non-Javadoc)
	 * @see net.morilib.lisp.Datum#getType()
	 */
	@Override
	public LispType getType() {
		return LispType.VECTOR;
	}
	
}
