/*
 * Copyright 2009 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.lisp.exlib;

import net.morilib.automata.nfa.NFABuildException;
import net.morilib.lisp.Datum;
import net.morilib.lisp.Environment;
import net.morilib.lisp.LispMessage;
import net.morilib.lisp.LispString;
import net.morilib.lisp.ParserSharpSyntax;
import net.morilib.lisp.Undef;
import net.morilib.lisp.subr.BinaryArgs;

public class DefineSharpSymbol extends BinaryArgs {

	@Override
	protected Datum execute(Datum c1a, Datum c2a, Environment env,
			LispMessage mesg) {
		if(c1a instanceof LispString) {
			ParserSharpSyntax pss = ParserSharpSyntax.getInstance();
			
			try {
				pss.addRule(
						((LispString)c1a).getString(),
						c2a, false, false);
				return Undef.UNDEF;
			} catch(NFABuildException e) {
				throw mesg.getError("err.pattern.syntax", c1a);
			}
		} else {
			throw mesg.getError("err.require.string", c1a);
		}
	}

}
